﻿/**********************************************************************\
**
**
**                          Copyright (c) 2017-2022 by
**                          HENGSTLER GmbH, Aldingen
**
**      This  software  is furnished under a license and may be used and
**      copied only in accordance with the terms  of  such  license  and
**      with the inclusion of the above copyright notice.  This software
**      or any other copies thereof may not  be  provided  or  otherwise
**      made  available  to any other person.  No title to and ownership
**      of the software is hereby transferred.
**
**      The  information  in  this software is subject to change without
**      notice and should not be construed as a commitment by Hengstler.
**
**  Module:      Exo-ApiWrapper_NET-Demo-MainForm.cs
**  Facility:    S684015 - Hengstler Extendo Printer Driver for Windows XP/7/8/10
**  Component:   Extendo Api-Wrapper .NET Demo Application
**  Version:     R10-V2.06-FINAL / 2022SEP28
**  Environment: Windows XP/7/8/10 / .NET Framework V4.0
**  Compiler:    Microsoft Visual Studio 2015
**  Abstract:    Handles main form
**  Note:        -
**  History:     R8-V2.04a-BETA / 2017JUN09 / HOR / M-R8-XXX-HOR 
**                  Extendo API Wrapper .NET demo application added 
**               R8-V2.04 / 2018JUL02 / HOR / 8TH RELEASE
**               -----------------------------------------------------
**               R9-V2.05 / 2021SEP14 / HOR / 9TH RELEASE
**               -----------------------------------------------------
**               R10-V2.06 / 2022SEP28 / HOR / 10TH RELEASE
**               -----------------------------------------------------
**
\**********************************************************************/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Exo_ApiWrapper_NET;
using System.Runtime.InteropServices;
using System.Threading;

namespace Exo_ApiWrapper_NET_Demo
{
    public partial class MainForm : Form
    {
       /*
        * CONSTANTS
        */

       /*
        * MEMBER ATTRIBUTES
        */

        ExoApiWrapper m_exoApi            = null;
        
        /*
         * METHODS
         */

        // -- Default Constructor
        public MainForm()
        {
            InitializeComponent();
        }
        
        // -- Method executed when form gets visible
        private void MainForm_Shown(object sender, EventArgs e)
        {
            // -- Try to init the Extendo C# API 
            // .. Load library Exo_Api.dll
            try
            {
                //-- Create new Extendo API Wrapper
                m_exoApi = new ExoApiWrapper();

                // -- Enumerate all Extendo printers and output names to combo box
                IntPtr hPrintEnum;
                if ((hPrintEnum = m_exoApi.Exo_Api_Pe_PrinterEnumStart()) != IntPtr.Zero)
                {
                    //-- Add all Extendo printers
                    string strPrinterName = string.Empty;
                    for (strPrinterName = m_exoApi.Exo_Api_Pe_PrinterEnum(hPrintEnum, true);
                        strPrinterName != null;
                        strPrinterName = m_exoApi.Exo_Api_Pe_PrinterEnum(hPrintEnum, false))
                    {
                        m_cmbPrinterList.Items.Add(strPrinterName);
                    }

                    //-- Select Extendo printer
                    if (m_cmbPrinterList.Items.Count > 0)
                        m_cmbPrinterList.SelectedIndex = 0;

                    //-- End Extendo print4er enumeration 
                    m_exoApi.Exo_Api_Pe_PrinterEnumEnd(hPrintEnum);
                }
            }
            catch (ExoApiException ex)
            {
                MessageBox.Show(ex.Message + " APPLICATION CANCELLED", "EXTENDO API WRAPPER FAULT", MessageBoxButtons.OK, MessageBoxIcon.Error);

                // -- Close form on error
                this.Close();
            }
                        
        }

        // -- Method executed when Form is closed
        private void MainForm_FormClosed(object sender, FormClosedEventArgs e)
        {
        }
                
        // -- Method fo connect to selected printer
        private IntPtr connectPrinter()
        {
            IntPtr printerHandle = IntPtr.Zero; // Printer handle

            // -- Open selected printer if valid printer name
            if (m_exoApi.Exo_Api_Pe_ValidatePrinterName(m_cmbPrinterList.SelectedItem.ToString()))
            {
                printerHandle = m_exoApi.Exo_Api_Pio_PrinterOpen(m_cmbPrinterList.SelectedItem.ToString(), ExoApiWrapper.D_EXO_API_PIO_OPT_NONE, 5000);
            }

            // -- Return handle of printer
            return printerHandle;
        }

        // -- Method for disconnecting from printer
        private void disconnectPrinter(IntPtr printerHandle)
        {
            // -- Close connection if  printer handle is non zero
            if (printerHandle != IntPtr.Zero)
            {
                // -- Close printer connection
                m_exoApi.Exo_Api_Pio_PrinterClose(printerHandle);
            }
        }

        // -- Button clicked method for inquiring printer status and output it to the printer data  text box
        private void m_btnInquirePrinterStatus_Click(object sender, EventArgs e)
        {
            String printerData   = "<NO DATA>"; // String containing printer data
          
            // -- Connect to printer 
            IntPtr printerHandle = IntPtr.Zero; // Printer handle
            if ((printerHandle = connectPrinter()) != IntPtr.Zero)
            {
                // -- Clear data reception buffer buffer
                m_exoApi.Exo_Api_Pio_PrinterReadClear(printerHandle);

                // -- Request printer status from printer
                ushort packetId       = ExoApiWrapper.D_EXO_API_DATA_REQUEST_ID_PRINTER_STATUS;
                byte[] printerCommand = { 0x1d, 0x61, (byte) (packetId & 0xFF)};
                if (m_exoApi.Exo_Api_Pio_PrinterWrite(printerHandle, printerCommand, (uint)printerCommand.Length, 100) == printerCommand.Length)
                {
                    // -- Process data transmitted by printer 
                    ExoApiWrapper.TS_EXO_API_PCKT_HEADER         packetHeader        = new ExoApiWrapper.TS_EXO_API_PCKT_HEADER();
                    ExoApiWrapper.TS_EXO_API_PCKT_PRINTER_STATUS packetPrinterStatus = new ExoApiWrapper.TS_EXO_API_PCKT_PRINTER_STATUS();
                    if (m_exoApi.Exo_Api_Pio_PrinterRead(printerHandle, packetHeader, packetPrinterStatus, 500) < 0)
                    {
                        // -- No data received
                        printerData = String.Format("Printer data reception fault: No data received");
                    }
                    else
                    // .. Validate packet identifier 
                    if (packetHeader.ubId != ExoApiWrapper.D_EXO_API_DATA_PACKET_ID_PRINTER_STATUS)
                    {
                        // -- Invalid packet identifier
                        printerData = String.Format( "Printer data reception fault: Invalid packet identifier ({0} instead of {1})"
                                                   , packetHeader.ubId
                                                   , ExoApiWrapper.D_EXO_API_DATA_PACKET_ID_PRINTER_STATUS);
                    }
                    else
                    // .. Verify packet length
                    if (packetHeader.ubLength != Marshal.SizeOf(packetPrinterStatus))
                    {
                        // -- Invalid packet length
                        printerData = String.Format( "Data reception fault: Packet length mismatch ({0} instead of {1})"
                                                   , packetHeader.ubLength
                                                   , Marshal.SizeOf(packetPrinterStatus));
                    }   
                    // .. Process printer data
                    else
                    {
                        //-- Process printer data
                        printerData = String.Format( "packetPrinterStatus.dwStatus                    = {0}\r\n"
                                                   + "packetPrinterStatus.ubPrintJobId                = {1}\r\n"
                                                   + "packetPrinterStatus.siPrintHeadTempC            = {2}\r\n"
                                                   + "packetPrinterStatus.wBoardVoltageTv             = {3}\r\n"
                                                   + "packetPrinterStatus.ubState                     = {4}\r\n"
                                                   + "packetPrinterStatus.wError                      = {5}\r\n"
                                                   , "[" + packetPrinterStatus.dwStatus.ToString("X04") + "]"
                                                   , packetPrinterStatus.ubPrintJobId.ToString()
                                                   , packetPrinterStatus.siPrintHeadTempC.ToString("+00;-00") + " °C"
                                                   , ((double)packetPrinterStatus.wBoardVoltageTv / 10).ToString("00.0") + " V"
                                                   , packetPrinterStatus.ubState.ToString()
                                                   , "[" + packetPrinterStatus.wError.ToString("X04") + "]"
                                                   );
                    }
                }

                //-- Disconnect printer
                disconnectPrinter(printerHandle);
            }

            //-- Output printer data to text box
            m_txtPrinterData.Text = printerData;
        }

        // -- Button clicked method for inquiring printer info and output it to the printer data text box
        private void m_btnInquirePrinterInfo_Click(object sender, EventArgs e)
        {
            String printerData   = "<NO DATA>"; // String containing printer data
          
            // -- Connect to printer 
            IntPtr printerHandle = IntPtr.Zero; // Printer handle
            if ((printerHandle = connectPrinter()) != IntPtr.Zero)
            {
                // -- Clear data reception buffer buffer
                m_exoApi.Exo_Api_Pio_PrinterReadClear(printerHandle);

                // -- Request printer info from printer
                ushort packetId       = ExoApiWrapper.D_EXO_API_DATA_REQUEST_ID_PRINTER_INFO;
                byte[] printerCommand = { 0x1d, 0x61, (byte) (packetId & 0xFF)};
                if (m_exoApi.Exo_Api_Pio_PrinterWrite(printerHandle, printerCommand, (uint)printerCommand.Length, 100) == printerCommand.Length)
                {
                    // -- Process data transmitted by printer 
                    ExoApiWrapper.TS_EXO_API_PCKT_HEADER       packetHeader        = new ExoApiWrapper.TS_EXO_API_PCKT_HEADER();
                    ExoApiWrapper.TS_EXO_API_PCKT_PRINTER_INFO packetPrinterInfo   = new ExoApiWrapper.TS_EXO_API_PCKT_PRINTER_INFO();
                    if (m_exoApi.Exo_Api_Pio_PrinterRead(printerHandle, packetHeader, packetPrinterInfo, 500) < 0)
                    {
                        // -- No data received
                        printerData = String.Format("Printer data reception fault: No data received");
                    }
                    else
                    // .. Validate packet identifier 
                    if (packetHeader.ubId != ExoApiWrapper.D_EXO_API_DATA_PACKET_ID_PRINTER_INFO)
                    {
                        // -- Invalid packet identifier
                        printerData = String.Format( "Printer data reception fault: Invalid packet identifier ({0} instead of {1})"
                                                   , packetHeader.ubId
                                                   , ExoApiWrapper.D_EXO_API_DATA_PACKET_ID_PRINTER_INFO);
                    }
                    else
                    // .. Verify packet length
                    if (packetHeader.ubLength != Marshal.SizeOf(packetPrinterInfo))
                    {
                        // -- Invalid packet length
                        printerData = String.Format( "Data reception fault: Packet length mismatch ({0} instead of {1})"
                                                   , packetHeader.ubLength
                                                   , Marshal.SizeOf(packetPrinterInfo));
                    }   
                    // .. Process printer data
                    else
                    {    
                        //-- Process printer data
                        printerData = String.Format( "packetPrinterInfo.ubProtocolRevision            = {0}\r\n"
                                                   + "packetPrinterInfo.szPartNr                      = {1}\r\n"
                                                   + "packetPrinterInfo.szSerialNr                    = {2}\r\n"
                                                   + "packetPrinterInfo.szDateOfManuf                 = {3}\r\n"
                                                   + "packetPrinterInfo.dwSysConfig                   = {4}\r\n"
                                                   + "packetPrinterInfo.dwSensorCode                  = {5}\r\n"
                                                   + "packetPrinterInfo.dwFeatureCode                 = {6}\r\n"
                                                   + "packetPrinterInfo.szFwPartNr                    = {7}\r\n"
                                                   + "packetPrinterInfo.szFwVersion                   = {8}\r\n"
                                                   + "packetPrinterInfo.szFwDate                      = {9}\r\n"
                                                   + "packetPrinterInfo.dwFwPatch                     = {10}\r\n"
                                                   , packetPrinterInfo.ubProtocolRevision.ToString()
                                                   , new String(packetPrinterInfo.szPartNr,      0, Array.FindIndex(packetPrinterInfo.szPartNr,      item => item == 0))               
                                                   , new String(packetPrinterInfo.szSerialNr,    0, Array.FindIndex(packetPrinterInfo.szSerialNr,    item => item == 0))               
                                                   , new String(packetPrinterInfo.szDateOfManuf, 0, Array.FindIndex(packetPrinterInfo.szDateOfManuf, item => item == 0))               
                                                   , "[" + packetPrinterInfo.dwSysConfig.ToString("X08") + "]"
                                                   , "[" + packetPrinterInfo.dwSysSensor.ToString("X08") + "]"
                                                   , "[" + packetPrinterInfo.dwSysFeature.ToString("X08") + "]"
                                                   , new String(packetPrinterInfo.szFwPartNr,   0, Array.FindIndex(packetPrinterInfo.szFwPartNr,  item => item == 0))               
                                                   , new String(packetPrinterInfo.szFwVersion,  0, Array.FindIndex(packetPrinterInfo.szFwVersion, item => item == 0))               
                                                   , new String(packetPrinterInfo.szFwDate,     0, Array.FindIndex(packetPrinterInfo.szFwDate,    item => item == 0))               
                                                   , "[" + packetPrinterInfo.dwFwPatch.ToString("X08") + "]"
                                                   );
                    }
                }

                //-- DisconnectPrinter the printer
                disconnectPrinter(printerHandle);
            }

            //-- Output printer data to text box
            m_txtPrinterData.Text = printerData;
        }

        // -- Button clicked method for inquiring sensor info and output it to the printer data text box
        private void m_btnInquirePrinterSensors_Click(object sender, EventArgs e)
        {
            String printerData   = "<NO DATA>"; // String containing printer data
          
            // -- Connect to printer 
            IntPtr printerHandle = IntPtr.Zero; // Printer handle
            if ((printerHandle = connectPrinter()) != IntPtr.Zero)
            {
                // -- Clear data reception buffer buffer
                m_exoApi.Exo_Api_Pio_PrinterReadClear(printerHandle);

                // -- Request printer status from printer
                ushort packetId = ExoApiWrapper.D_EXO_API_DATA_REQUEST_ID_PRINTER_SENSORS;
                byte[] printerCommand = { 0x1d, 0x61, (byte) (packetId & 0xFF)};
                if (m_exoApi.Exo_Api_Pio_PrinterWrite(printerHandle, printerCommand, (uint)printerCommand.Length, 100) == printerCommand.Length)
                {
                    // -- Process data transmitted by printer 
                    ExoApiWrapper.TS_EXO_API_PCKT_HEADER         packetHeader        = new ExoApiWrapper.TS_EXO_API_PCKT_HEADER();
                    ExoApiWrapper.TS_EXO_API_PCKT_PRINTER_SENSOR packetPrinterSensor = new ExoApiWrapper.TS_EXO_API_PCKT_PRINTER_SENSOR();
                    if (m_exoApi.Exo_Api_Pio_PrinterRead(printerHandle, packetHeader, packetPrinterSensor, 500) < 0)
                    {
                        // -- No data received
                        printerData = String.Format("Printer data reception fault: No data received");
                    }
                    else
                    // .. Validate packet identifier 
                    if (packetHeader.ubId != ExoApiWrapper.D_EXO_API_DATA_PACKET_ID_PRINTER_SENSORS)
                    {
                        // -- Invalid packet identifier
                        printerData = String.Format( "Printer data reception fault: Invalid packet identifier ({0} instead of {1})"
                                                   , packetHeader.ubId
                                                   , ExoApiWrapper.D_EXO_API_DATA_PACKET_ID_PRINTER_SENSORS);
                    }
                    else
                    // .. Verify packet length
                    if (packetHeader.ubLength != Marshal.SizeOf(packetPrinterSensor))
                    {
                        // -- Invalid packet length
                        printerData = String.Format( "Data reception fault: Packet length mismatch ({0} instead of {1})"
                                                   , packetHeader.ubLength
                                                   , Marshal.SizeOf(packetPrinterSensor));
                    }   
                    // .. Process printer data
                    else
                    {    
                        //-- Process printer data
                        printerData = String.Format( "packetPrinterSensor.ubProtocolRevision          = {0}\r\n"
                                                   + "packetPrinterSensor.ubAdcMeasurement            = {1}\r\n"
                                                   + "packetPrinterSensor.wDigitalInput               = {2}\r\n"
                                                   + "packetPrinterSensor.siPrintHeadTempC            = {3}\r\n"
                                                   + "packetPrinterSensor.wBoardVoltageTv             = {4}\r\n"
                                                   + "packetPrinterSensor.ubPaperPreEndSensorState    = {5}\r\n"
                                                   + "packetPrinterSensor.ubPaperEntrySensor1State    = {6}\r\n"
                                                   + "packetPrinterSensor.ubPaperEntrySensor2State    = {7}\r\n"
                                                   + "packetPrinterSensor.ubHeadUpSensorState         = {8}\r\n"
                                                   + "packetPrinterSensor.ubCutterSensor1State        = {9}\r\n"
                                                   + "packetPrinterSensor.ubCutterSensor1State        = {10}\r\n"
                                                   + "packetPrinterSensor.ubPaperExitSensorState      = {11}\r\n"
                                                   + "packetPrinterSensor.ubAuxiliarySensorState      = {12}\r\n"
                                                   , packetPrinterSensor.ubProtocolRevision.ToString()
                                                   , "[" + BitConverter.ToString(packetPrinterSensor.ubAdcMeasurement).Replace("-", "]/[") + "]"
                                                   , "[" + packetPrinterSensor.wDigitalInput.ToString("X04")  + "]"
                                                   , packetPrinterSensor.siPrintHeadTempC.ToString("+00;-00") + " °C"
                                                   , ((double)packetPrinterSensor.wBoardVoltageTv / 10).ToString("00.0") + " V"
                                                   , "[" + packetPrinterSensor.ubPaperPreEndSensorState.ToString("X02") + "]"
                                                   , "[" + packetPrinterSensor.ubPaperEntrySensor1State.ToString("X02") + "]"
                                                   , "[" + packetPrinterSensor.ubPaperEntrySensor2State.ToString("X02") + "]"
                                                   , "[" + packetPrinterSensor.ubHeadUpSensorState.ToString("X02") + "]"
                                                   , "[" + packetPrinterSensor.ubCutterSensor1State.ToString("X02") + "]"
                                                   , "[" + packetPrinterSensor.ubCutterSensor2State.ToString("X02") + "]"
                                                   , "[" + packetPrinterSensor.ubPaperExitSensorState.ToString("X02") + "]"
                                                   , "[" + packetPrinterSensor.ubAuxiliarySensorState.ToString("X02") + "]"
                                                   );
                    }
                }

                //-- DisconnectPrinter the printer
                disconnectPrinter(printerHandle);
            }

            //-- Output printer data to text box
            m_txtPrinterData.Text = printerData;

        }

        // -- Button clicked method for inquiring life cycle data and output it to the printer data text box
        private void m_btnInquireLifeCycle_Click(object sender, EventArgs e)
        {
            String printerData   = "<NO DATA>"; // String containing printer data
          
            // -- Connect to printer 
            IntPtr printerHandle = IntPtr.Zero; // Printer handle
            if ((printerHandle = connectPrinter()) != IntPtr.Zero)
            {
                // -- Clear data reception buffer buffer
                m_exoApi.Exo_Api_Pio_PrinterReadClear(printerHandle);

                // -- Request life cycle data from printer
                ushort packetId       = ExoApiWrapper.D_EXO_API_DATA_REQUEST_ID_PRINTER_LIFE_CYCLE;
                byte[] printerCommand = { 0x1d, 0x61, (byte) (packetId & 0xFF)};
                if (m_exoApi.Exo_Api_Pio_PrinterWrite(printerHandle, printerCommand, (uint)printerCommand.Length, 100) == printerCommand.Length)
                {
                    // -- Process data transmitted by printer 
                    ExoApiWrapper.TS_EXO_API_PCKT_HEADER              packetHeader    = new ExoApiWrapper.TS_EXO_API_PCKT_HEADER();
                    ExoApiWrapper.TS_EXO_API_PCKT_PRINTER_LIFE_CYCLE  packetLifeCycle = new ExoApiWrapper.TS_EXO_API_PCKT_PRINTER_LIFE_CYCLE();
                    if (m_exoApi.Exo_Api_Pio_PrinterRead(printerHandle, packetHeader, packetLifeCycle, 500) < 0)
                    {
                        // -- No data received
                        printerData = String.Format("Printer data reception fault: No data received");
                    }
                    else
                    // .. Validate packet identifier 
                    if (packetHeader.ubId != ExoApiWrapper.D_EXO_API_DATA_PACKET_ID_PRINTER_LIFE_CYCLE)
                    {
                        // -- Invalid packet identifier
                        printerData = String.Format( "Printer data reception fault: Invalid packet identifier ({0} instead of {1})"
                                                   , packetHeader.ubId
                                                   , ExoApiWrapper.D_EXO_API_DATA_PACKET_ID_PRINTER_LIFE_CYCLE);
                    }
                    else
                    // .. Verify packet length
                    if (packetHeader.ubLength != Marshal.SizeOf(packetLifeCycle))
                    {
                        // -- Invalid packet length
                        printerData = String.Format( "Data reception fault: Packet length mismatch ({0} instead of {1})"
                                                   , packetHeader.ubLength
                                                   , Marshal.SizeOf(packetLifeCycle));
                    }   
                    // .. Process printer data
                    else
                    {    
                        //-- Process printer data
                        printerData = String.Format( "packetLifeCycle.ubProtocolRevision              = {0}\r\n"
                                                   + "packetLifeCycle.dwTimePoweredS                  = {1}\r\n"
                                                   + "packetLifeCycle.dwDistanceSteppedMm             = {2}\r\n"
                                                   + "packetLifeCycle.dwDistanceBurnedMm              = {3}\r\n"
                                                   + "packetLifeCycle.dwPaperTransportedMm            = {4}\r\n"
                                                   + "packetLifeCycle.dwNrOfSystemStarts              = {5}\r\n"
                                                   + "packetLifeCycle.dwNrOfHardResets                = {6}\r\n"
                                                   + "packetLifeCycle.dwNrOfPaperCuts                 = {7}\r\n"
                                                   + "packetLifeCycle.wNrOfPaperLoads                 = {8}\r\n"
                                                   , packetLifeCycle.ubProtocolRevision.ToString()
                                                   , packetLifeCycle.dwTimePoweredS.ToString("0000000000") + " s"
                                                   , packetLifeCycle.dwDistanceSteppedMm.ToString("0000000000")  + " mm"
                                                   , packetLifeCycle.dwDistanceBurnedMm.ToString("0000000000")  + " mm"
                                                   , packetLifeCycle.dwPaperTransportedMm.ToString("0000000000")  + " mm"
                                                   , packetLifeCycle.dwNrOfSystemStarts.ToString("0000000000") 
                                                   , packetLifeCycle.dwNrOfHardResets.ToString("0000000000")
                                                   , packetLifeCycle.dwNrOfPaperCuts.ToString("0000000000")
                                                   , packetLifeCycle.wNrOfPaperLoads.ToString("0000000000")
                                                   );
                    }
                }

                //-- DisconnectPrinter the printer
                disconnectPrinter(printerHandle);
            }

            //-- Output printer data to text box
            m_txtPrinterData.Text = printerData;
        }

        // -- Button for printing data received from printer
        private void m_btnPrintData_Click(object sender, EventArgs e)
        {
            String printerData   = "<NO DATA>"; // String containing printer data
          
            //-- Output printer data to text box
            if (m_txtPrinterData.Text.Count() > 0)
               printerData = m_txtPrinterData.Text;

            // -- Connect to printer 
            IntPtr printerHandle = IntPtr.Zero; // Printer handle
            if ((printerHandle = connectPrinter()) != IntPtr.Zero)
            {
                // -- Create print buffer
                char [] printBuffer = new char[0]; 

                // -- Add soft reset the print buffer
                byte[] cmdSoftReset=  { 0x1B, 0x40 };
                Array.Resize(ref printBuffer, printBuffer.Length + cmdSoftReset.Length);
                cmdSoftReset.CopyTo(printBuffer, printBuffer.Length - cmdSoftReset.Length);
               
                // -- Add printer data to print buffer
                Array.Resize(ref printBuffer, printBuffer.Length + printerData.Length);
                Encoding.ASCII.GetBytes(printerData).CopyTo(printBuffer, printBuffer.Length - printerData.Length);

                // -- Add form feed to print buffer
                byte[] cmdFormFeed =  { 0x0C };
                Array.Resize(ref printBuffer, printBuffer.Length + cmdFormFeed.Length);
                cmdFormFeed.CopyTo(printBuffer, printBuffer.Length - cmdFormFeed.Length);

                // -- Add full cut command to print buffer
                byte[] cmdFullCut =  { 0x1B, 0xF0, 0x06, 0x01, 0x02 };
                Array.Resize(ref printBuffer, printBuffer.Length + cmdFullCut.Length);
                cmdFullCut.CopyTo(printBuffer, printBuffer.Length - cmdFullCut.Length);

                //-- Write the print buffer
                m_exoApi.Exo_Api_Pio_PrinterWrite(printerHandle, printBuffer, (uint) printBuffer.Length, 5000);

                //-- DisconnectPrinter the printer
                disconnectPrinter(printerHandle);
            }
        }

        // -- Print image on printer
        private void m_btnPrintImage_Click(object sender, EventArgs e)
        { 
            // -- Show open file dialog for 
            OpenFileDialog openFileDialog = new OpenFileDialog();
            openFileDialog.Filter = "Image file (*.jpg;*.bmp;*.png;*.gif;*.tiff;*.tif)|*.jpg;*.bmp;*.png;*.gif;*.tiff;*.tif|All files (*.*)|*.*";
            if (openFileDialog.ShowDialog() == DialogResult.OK) {
     
                // -- Connect to printer 
                IntPtr printerHandle = IntPtr.Zero; // Printer handle
                if ((printerHandle = connectPrinter()) != IntPtr.Zero)
                { 
                    // -- Print image on printer
                    m_exoApi.Exo_Api_Pio_PrinterPrintImage(printerHandle, openFileDialog.FileName, 5000);

                    // -- Send full cut command to print buffer
                    byte[] cmdFullCut =  { 0x1B, 0xF0, 0x06, 0x01, 0x02 };
                    m_exoApi.Exo_Api_Pio_PrinterWrite(printerHandle, cmdFullCut, (uint) cmdFullCut.Length, 5000);

                    //-- DisconnectPrinter the printer
                    disconnectPrinter(printerHandle); 
                }
            }
        }

        // -- Exit button 
        private void m_btnExit_Click(object sender, EventArgs e)
        {
            // -- Close form
            Close();
        }      
    }
}
