/**********************************************************************\
**
**                         Copyright (c) 2007-2022 by
**                          HENGSTLER GmbH, Aldingen 
**
**      This  software  is furnished under a license and may be used and
**      copied only in accordance with the terms  of  such  license  and
**      with the inclusion of the above copyright notice.  This software
**      or any other copies thereof may not  be  provided  or otherwise
**      made  available  to any other person.  No title to and ownership
**      of the software is hereby transferred.
**
**      The  information  in  this software is subject to change without
**      notice and should not be construed as a commitment by Hengstler.
**
**  Module:      Exo-Pmpt-DlgMain.cpp
**  Facility:    S684015 - Hengstler Extendo Printer Driver for Windows XP/7/8/10 
**  Component:   Extendo print monitor plugin test tool (Exo_PrintMonPluginTest)
**  Version:     R10-V2.06-FINAL / 2022SEP28
**  Environment: Microsoft Windows XP/7/8/10
**  Compiler:    Microsoft Visual C++ .NET   
**  Abstract:    Handler of main dialog
**  Note:        -
**  History:     R1-V1.00 / 2007DEC21 / HOR / 1ST RELEASE 
**               -----------------------------------------------------
**               R2-V1.01 / 2008MAY31 / HOR / 2ND RELEASE 
**               -----------------------------------------------------
**               R3-V1.02 / 2010APR30 / HOR / 3RD RELEASE
**               -----------------------------------------------------
**               R4-V2.00a  / 2011MAY26 / HOR / M-R4-004-HOR
**                  Complete printer package ported to VS2010, reviewed and adapted Windows 7 32+64 bit
**               R4-V2.00 / 2013JUL18 / HOR / 4TH RELEASE
**               -----------------------------------------------------
**               R5-V2.01 / 2015MAY07 / HOR / 5TH RELEASE
**               -----------------------------------------------------
**               R6-V2.02b / 2016MAY25 / HOR / M-R6-007-HOR
**                  Driver package ported to VS2015 and adapted to Windows 10.
**               R6-V2.02 / 2016MAY25 / HOR / 6TH RELEASE
**               -----------------------------------------------------
**               R7-V2.03 / 2017APR20 / HOR / 7TH RELEASE
**               -----------------------------------------------------
**               R8-V2.04 / 2018JUL02 / HOR / 8TH RELEASE
**               -----------------------------------------------------
**               R9-V2.05 / 2021SEP14 / HOR / 9TH RELEASE
**               -----------------------------------------------------
**               R10-V2.06 / 2022SEP28 / HOR / 10TH RELEASE
**               -----------------------------------------------------
**
\**********************************************************************/

/*
** INCLUDES
*/

#include "stdafx.h"
#include "winspool.h"
#include "Exo_PrintMonPlugin.h"
#include "Exo_Api.h"
#include "Exo-PrintMonPluginTest.h"
#include "Exo-Pmpt-DlgMain.h"

/*
** COMPILER 
*/

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

/*
** MACROS
*/

/*
** TYPES
*/

/*
** FUNCTIONS
*/

/*
** VARIABLES
*/

/* 
** MESSAGE MAPS
*/

BEGIN_MESSAGE_MAP(CExo_Pmpt_DlgMain, CDialog)
   ON_WM_SYSCOMMAND()
   ON_WM_PAINT()
   ON_WM_QUERYDRAGICON()
   //}}AFX_MSG_MAP
   ON_WM_TIMER()
   ON_CBN_SELCHANGE(IDC_COMBO_PRINTER_DRIVER, OnCbnSelchangeComboPrinterDriver)
END_MESSAGE_MAP()

//******************************************************************************************
//** METHOD:   CExo_Pmpt_DlgMain::CExo_Pmpt_DlgMain
//** ABSTRACT: Constructor
//******************************************************************************************

CExo_Pmpt_DlgMain::CExo_Pmpt_DlgMain
   (CWnd* pParent /*=NULL*/)
   : CDialog(CExo_Pmpt_DlgMain::IDD, pParent)
{
   //-- Load icons
   m_hIconL  =AfxGetApp()->LoadIcon(IDR_ICO_MAIN_L);
   m_hIconS = AfxGetApp()->LoadIcon(IDR_ICO_MAIN_S);

   //-- Initialize variables
   m_iTimer     = 0;
   m_pszPrinter = NULL;
}

//******************************************************************************************
//** METHOD:   CExo_Pmpt_DlgMain::~CExo_Pmpt_DlgMain
//** ABSTRACT: Destructor
//******************************************************************************************

CExo_Pmpt_DlgMain::~CExo_Pmpt_DlgMain
   ()
{
   //-- Free printer if allocated
   if (m_pszPrinter) free(m_pszPrinter); m_pszPrinter= NULL;
}

//******************************************************************************************
//** METHOD:   CExo_Pmpt_DlgMain::DoDataExchange
//** ABSTRACT: This method is called by the framework to exchange and validate dialog data.
//******************************************************************************************

void CExo_Pmpt_DlgMain::DoDataExchange
   (CDataExchange* pDX)
{
   //-- ...
   CDialog::DoDataExchange(pDX);
}

//******************************************************************************************
//** METHOD:   CExo_Pmpt_DlgMain::OnInitDialog
//** ABSTRACT: This method is called by the framework in response to the WM_INITDIALOG message
//******************************************************************************************

BOOL CExo_Pmpt_DlgMain::OnInitDialog
   ()
{
   //-- Init dialog
   CDialog::OnInitDialog();

   //-- Set icons
   SetIcon(m_hIconL, TRUE);   
   SetIcon(m_hIconS, FALSE);  

   //-- Modify system menu and add dialogs
   CMenu* pSysMenu = this->GetSystemMenu(FALSE);

   //-- Setup combo box: printer driver
   { 
      CComboBox * poComboBox;   // Pointer to combo box
 
      //-- Get pointer to combo box
      if ((poComboBox = ((CComboBox *) this->GetDlgItem(IDC_COMBO_PRINTER_DRIVER))) != NULL) {

         HANDLE hPrinterEnum;  // Handle of printer enumeration

         //-- Reset contents of box
         poComboBox->ResetContent() ;

         //-- Start printer enumeration
         if ((hPrinterEnum = Exo_Api_Pe_PrinterEnumStart()) != NULL) {

            PSTR pszPrinterName;  // Name of printer

            //-- Go through all printers
            for (pszPrinterName = Exo_Api_Pe_PrinterEnum(hPrinterEnum, true); pszPrinterName; pszPrinterName = Exo_Api_Pe_PrinterEnum(hPrinterEnum, false)) {
 
                //-- Add printer
                poComboBox->AddString(pszPrinterName);
            }

            //-- End of printer enumeration
            Exo_Api_Pe_PrinterEnumEnd(hPrinterEnum);
         }
 
         //-- Select printer
         poComboBox->SetCurSel(0);
      }
   }
   
   //-- Get current driver name 
   OnCbnSelchangeComboPrinterDriver();

   //-- Update dialog
   UpdateDialog();

   //-- Setup update timer to .. hz
   m_iTimer = SetTimer(1, 1000, NULL);

   //-- All done
   return TRUE; 
}

//******************************************************************************************
//** METHOD:   CExo_Pmpt_DlgMain::OnSysCommand
//** ABSTRACT: This method is called by the framework when the user selects a command from 
//**          the Control menu, or when the user selects the Maximize or the Minimize button.
//******************************************************************************************

void CExo_Pmpt_DlgMain::OnSysCommand
   (UINT nID, LPARAM lParam)
{
    CDialog::OnSysCommand(nID, lParam);
}

//******************************************************************************************
//** METHOD:   CExo_Pmpt_DlgMain::OnPaint
//** ABSTRACT: This method is called by the framework when an application makes a request to 
//**          repaint a portion of an application window. 
//******************************************************************************************

void CExo_Pmpt_DlgMain::OnPaint
   () 
{
   if (IsIconic()) {

      CPaintDC dc(this); //--Gertekontext zum Zeichnen

      SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

      //-- Symbol in Clientrechteck zentrieren
      int cxIcon = GetSystemMetrics(SM_CXICON);
      int cyIcon = GetSystemMetrics(SM_CYICON);
      CRect rect;
      GetClientRect(&rect);
      int x = (rect.Width() - cxIcon + 1) / 2;
      int y = (rect.Height() - cyIcon + 1) / 2;

      //-- Symbol zeichnen
      dc.DrawIcon(x, y, this->m_hIconS);
   }
   else {
      CDialog::OnPaint();
   }
}

//******************************************************************************************
//** METHOD:   CExo_Pmpt_DlgMain::OnQueryDragIcon
//** ABSTRACT: The framework calls this member function by a minimized (iconic) window that 
//**          does not have an icon defined for its class
//******************************************************************************************

HCURSOR CExo_Pmpt_DlgMain::OnQueryDragIcon
   ()
{
   return static_cast<HCURSOR>(this->m_hIconS);
}

//******************************************************************************************
//** METHOD     CExo_Pmpt_DlgMain::OnTimer
//** ABSTRACT   Cyclic timer event (1 sec ) to update structures from registry and 
//             dialog elements
//******************************************************************************************

void CExo_Pmpt_DlgMain::OnTimer
   (UINT nIDEvent)
{
   //-- Update dialog
   UpdateDialog();
}

//******************************************************************************************
//** METHOD     CExo_Pmpt_DlgMain::OnCbnSelchangeComboPrinterDriver
//** ABSTRACT   Handles printer driver selection combo box
//******************************************************************************************

void CExo_Pmpt_DlgMain::OnCbnSelchangeComboPrinterDriver
   ()
{
   //-- Free printer if allocated
   if (m_pszPrinter) free(m_pszPrinter); m_pszPrinter= NULL;

   //-- Get combo box selection: printer driver
   { 
      CComboBox * poComboBox;   // Pointer to combo box
      INT         iSel;         // Selected item
 
      //-- Get pointer to combo box
      if ((poComboBox = ((CComboBox *) this->GetDlgItem(IDC_COMBO_PRINTER_DRIVER))) != NULL) {

         //-- Get selected item
         if ((iSel = poComboBox->GetCurSel()) != CB_ERR) {

            //-- Allocate memory for printer name 
            if ((m_pszPrinter = (PSTR) malloc(poComboBox->GetLBTextLen(iSel)+1)) != NULL) {

               //-- Get text of selected item
               poComboBox->GetLBText(iSel, m_pszPrinter);
            }
         }
      }
   }

   //-- Update dialog
   UpdateDialog();
}

//******************************************************************************************
//** METHOD     CExo_Pmpt_DlgMain::ReadPrinterDataFromRegistry
//** ABSTRACT   Reads printer data from registry
//******************************************************************************************

bool CExo_Pmpt_DlgMain::ReadPrinterDataFromRegistry
   (PSTR pszPrinter, PSTR pszKey, PSTR pszValue, PBYTE pubData, DWORD dwSize)
{
   bool                            fResult                    = false;  // Return value, FALSE by default
   PRINTER_DEFAULTS                sPrinterDefaults;                    // Printer defaults structure
   HANDLE                          hPrinter                   = NULL;   // Handle of printer

   //-- Validate parameter
   if (!pszPrinter || !pszKey || !pszValue || !pubData || !dwSize)
      goto FAILURE;

   //-- Setup printer defaults
   ZeroMemory(&sPrinterDefaults, sizeof(sPrinterDefaults));
   sPrinterDefaults.DesiredAccess = PRINTER_ACCESS_USE;
  
   //-- Open printer 
   if (!OpenPrinter(pszPrinter, &hPrinter, &sPrinterDefaults)) 
      goto FAILURE;

   //-- Read data from registery and reset data if not (yet) available or if revision nr differs
   {
      DWORD dwType;   //--Type of registry data
      DWORD dwNeeded; //--Bytes needed
   
       if (    (GetPrinterDataExA(hPrinter, pszKey, pszValue, &dwType, pubData, dwSize, &dwNeeded)  != ERROR_SUCCESS) 
            || (dwType != REG_BINARY)
          ) {
            goto FAILURE;
      }
   }

   //-- Close printer
   ClosePrinter(hPrinter); hPrinter = NULL;

   //-- Success
   fResult = true;

   //-- All done
   goto EXIT;

FAILURE:
   ;

EXIT:

   //-- Close printer if open
   if (hPrinter) ClosePrinter(hPrinter); hPrinter = NULL;

   //-- Return TRUE on success and FALSE otherwise
   return(fResult);
}

//************************************************************************
//** METHOD   CExo_Pmpt_DlgMain::UpdateJobInfoInDialog
//** ABSTRACT Updates job information in the main dialog
//************************************************************************

void CExo_Pmpt_DlgMain::UpdateJobInfoInDialog
   (PS_EXO_PMP_JOB_INFO psJobInfo)
{
   DWORD                dwAgeSec   = -1;        // Age of job information in sec
   CString              strItem;                // A string

   //-- Update registry path to job information
   {
      //-- Path is ...\KEY\VALUE
      strItem += "...\\";
      strItem += D_EXO_PMP_REG_KEY;
      strItem += "\\";
      strItem += D_EXO_PMP_REG_VALUE_PMP_JOB_INFO;
      GetDlgItem(IDC_TXT_JI_REGISTRY_ENTRY)->SetWindowText(strItem);
   }

   //-- Determine age of job information if valid
   if (psJobInfo && (psJobInfo->dwTsLastUpdate != (DWORD) -1)) 
        dwAgeSec = (GetTickCount() - psJobInfo->dwTsLastUpdate) / 1000;
   else dwAgeSec = (DWORD) -1; 

   //-- Update age of job information
   {
      //-- Regard update timestamp valid if ...
      //..     job info is valid 
      //.. AND age differs from 0xFFFFFFFF
      if (psJobInfo && (dwAgeSec != (DWORD) -1)) 
           strItem.Format("%u sec", dwAgeSec);
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_JI_TIME_SINCE_LAST_UPDATE)->SetWindowText(strItem);
   }

   //-- Update job identifier
   { 
      //-- Regard job identifier valid if ...
      //..     job info is valid 
      //.. AND age less than ... ms 
      if (psJobInfo && (dwAgeSec < 99999)) {
         if (psJobInfo->dwJobId != (DWORD) -1)
              strItem.Format("%d", psJobInfo->dwJobId);
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_JI_JOB_IDENTIFIER)->SetWindowText(strItem);
   }

   //-- Update job status
   { 
      //-- Regard job status valid if ...
      //..     job info is valid 
      //.. AND age less than ... ms 
      if (psJobInfo && (dwAgeSec < 99999)) {
         if (psJobInfo->dwJobId != (DWORD) -1)
              strItem.Format("%d", psJobInfo->dwJobStatus);
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_JI_JOB_STATUS)->SetWindowText(strItem);
   }

   //-- Update page count 
   {
      //-- Regard page count valid if ...
      //..     job info is valid 
      //.. AND age less than ... ms 
      if (psJobInfo && (dwAgeSec < 99999)) {
         if (psJobInfo->dwJobId != (DWORD) -1)
              strItem.Format("%d", psJobInfo->dwPageCount);
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_JI_PAGE_COUNT)->SetWindowText(strItem);
   }
}

//************************************************************************
//** METHOD   CExo_Pmpt_DlgMain::UpdatePrinterInfoInDialog
//** ABSTRACT Updates printer information in the main dialog
//************************************************************************

void CExo_Pmpt_DlgMain::UpdatePrinterInfoInDialog
   (PS_EXO_PMP_PRINTER_INFO psPrinterInfo)
{
   DWORD                dwAgeSec   = -1;        // Age of printer information in seconds
   CString              strItem;                // A string

   //-- Update registry path to printer information
   {
      //-- Path is ...\KEY\VALUE
      strItem += "...\\";
      strItem += D_EXO_PMP_REG_KEY;
      strItem += "\\";
      strItem += D_EXO_PMP_REG_VALUE_PMP_PRINTER_INFO;
      GetDlgItem(IDC_TXT_PI_REGISTRY_ENTRY)->SetWindowText(strItem);
   }

   //-- Determine age of printer information if valid 
   if (psPrinterInfo && (psPrinterInfo->dwTsLastUpdate != (DWORD) -1)) 
        dwAgeSec = (GetTickCount() - psPrinterInfo->dwTsLastUpdate) / 1000;
   else dwAgeSec = (DWORD) -1; 

   //-- Update age of printer information
   {
      //-- Regard update timestamp valid if ...
      //..     printer info is valid 
      //.. AND age differs from 0xFFFFFFFF
      if (psPrinterInfo && (dwAgeSec != (DWORD) -1))
           strItem.Format("%u sec", dwAgeSec);
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PI_TIME_SINCE_LAST_UPDATE)->SetWindowText(strItem);
   }

   //-- Update communication protocol revision
   { 
      //-- Regard communication protocol revision valid if ...
      //..     printer info is valid 
      //.. AND age less than ... ms 
      if (psPrinterInfo && (dwAgeSec < 99999)) {
         if (psPrinterInfo->dwDataValidFlag & D_EXO_PMP_PI_DVF_PRINTER_INFO) 
              strItem.Format("%d", psPrinterInfo->sPrinterInfo.ubProtocolRevision);
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PI_COMMUNICATION_PROTOCOL_REVISION)->SetWindowText(strItem);
   }

   //-- Update printer part number 
   { 
      //-- Regard printer part number valid if ...
      //..     printer info is valid 
      //.. AND age less than ... ms 
      if (psPrinterInfo && (dwAgeSec < 99999)) {
         if (    (psPrinterInfo->dwDataValidFlag & D_EXO_PMP_PI_DVF_PRINTER_INFO) 
              && psPrinterInfo->sPrinterInfo.szPartNr[0]
            ) strItem = psPrinterInfo->sPrinterInfo.szPartNr;
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PI_PART_NUMBER)->SetWindowText(strItem);
   }

   //-- Update printer serial number 
   { 
      //-- Regard printer serial number valid if ...
      //..     printer info is valid 
      //.. AND age less than ... ms 
      if (psPrinterInfo && (dwAgeSec < 99999)) {
         if (    (psPrinterInfo->dwDataValidFlag & D_EXO_PMP_PI_DVF_PRINTER_INFO) 
              && psPrinterInfo->sPrinterInfo.szSerialNr[0]
            ) strItem = psPrinterInfo->sPrinterInfo.szSerialNr;
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PI_SERIAL_NUMBER)->SetWindowText(strItem);
   }

   //-- Update date of manufacturing 
   { 
      //-- Regard date of manufacturing valid if ...
      //..     printer info is valid 
      //.. AND age less than ... ms 
      if (psPrinterInfo && (dwAgeSec < 99999)) {
         if (    (psPrinterInfo->dwDataValidFlag & D_EXO_PMP_PI_DVF_PRINTER_INFO) 
              && psPrinterInfo->sPrinterInfo.szDateOfManuf[0]
            ) strItem = psPrinterInfo->sPrinterInfo.szDateOfManuf;
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PI_DATE_OF_MANUFACTURING)->SetWindowText(strItem);
   }

   //-- Update configuration code
   { 
      //-- Regard configuration code valid if ...
      //..     printer info is valid 
      //.. AND age less than ... ms 
      if (psPrinterInfo && (dwAgeSec < 99999)) {
         if (psPrinterInfo->dwDataValidFlag & D_EXO_PMP_PI_DVF_PRINTER_INFO) 
              strItem.Format("[%8.8x]", psPrinterInfo->sPrinterInfo.dwSysConfig);
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PI_CONFIGURATION_CODE)->SetWindowText(strItem);
   }

   //-- Update sensor code
   { 
      //-- Regard sensor code valid if ...
      //..     printer info is valid 
      //.. AND age less than ... ms 
      if (psPrinterInfo && (dwAgeSec < 99999)) {
         if (psPrinterInfo->dwDataValidFlag & D_EXO_PMP_PI_DVF_PRINTER_INFO) 
              strItem.Format("[%8.8x]", psPrinterInfo->sPrinterInfo.dwSysSensor);
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PI_SENSOR_CODE)->SetWindowText(strItem);
   }

   //-- Update feature code
   { 
      //-- Regard feature code valid if ...
      //..     printer info is valid 
      //.. AND age less than ... ms 
      if (psPrinterInfo && (dwAgeSec < 99999)) {
         if (psPrinterInfo->dwDataValidFlag & D_EXO_PMP_PI_DVF_PRINTER_INFO) 
              strItem.Format("[%8.8x]", psPrinterInfo->sPrinterInfo.dwSysFeature);
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PI_FEATURE_CODE)->SetWindowText(strItem);
   }

   //-- Update firmware part number
   { 
      //-- Regard firmware part number valid if ...
      //..     printer info is valid 
      //.. AND age less than ... ms 
      if (psPrinterInfo && (dwAgeSec < 99999)) {
         if (psPrinterInfo->dwDataValidFlag & D_EXO_PMP_PI_DVF_PRINTER_INFO) 
              strItem = psPrinterInfo->sPrinterInfo.szFwPartNr;
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PI_FIRMWARE_PART_NR)->SetWindowText(strItem);
   }

   //-- Update firmware version
   { 
      //-- Regard firmware version valid if ...
      //..     printer info is valid 
      //.. AND age less than ... ms 
      if (psPrinterInfo && (dwAgeSec < 99999)) {
         if (psPrinterInfo->dwDataValidFlag & D_EXO_PMP_PI_DVF_PRINTER_INFO) 
              strItem = psPrinterInfo->sPrinterInfo.szFwVersion;
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PI_FIRMWARE_VERSION)->SetWindowText(strItem);
   }

   //-- Update firmware date
   { 
      //-- Regard firmware date valid if ...
      //..     printer info is valid 
      //.. AND age less than ... ms 
      if (psPrinterInfo && (dwAgeSec < 99999)) {
         if (psPrinterInfo->dwDataValidFlag & D_EXO_PMP_PI_DVF_PRINTER_INFO) 
              strItem = psPrinterInfo->sPrinterInfo.szFwDate;
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PI_FIRMWARE_DATE)->SetWindowText(strItem);
   }
}

//************************************************************************
//** METHOD   CExo_Pmpt_DlgMain::UpdatePrinterStatusInDialog
//** ABSTRACT Updates status information in the main dialog
//************************************************************************

void CExo_Pmpt_DlgMain::UpdatePrinterStatusInDialog
   (PS_EXO_PMP_PRINTER_STATUS psPrinterStatus)
{
   DWORD                dwAgeSec   = -1;        // Age of status information in seconds
   CString              strItem;                // A string

   //-- Update registry path to printer information
   {
      //-- Path is ...\KEY\VALUE
      strItem += "...\\";
      strItem += D_EXO_PMP_REG_KEY;
      strItem += "\\";
      strItem += D_EXO_PMP_REG_VALUE_PMP_PRINTER_STATUS;
      GetDlgItem(IDC_TXT_SI_REGISTRY_ENTRY)->SetWindowText(strItem);
   }

   //-- Determine age of status information if valid 
   if (psPrinterStatus && (psPrinterStatus->dwTsLastUpdate != (DWORD) -1)) 
        dwAgeSec = (GetTickCount() - psPrinterStatus->dwTsLastUpdate) / 1000;
   else dwAgeSec = (DWORD) -1; 

   //-- Update age of printer information
   {
      //-- Regard update timestamp valid if ...
      //..     printer info is valid 
      //.. AND age differs from 0xFFFFFFFF
      if (psPrinterStatus && (dwAgeSec != (DWORD) -1))
           strItem.Format("%u sec", dwAgeSec);
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_SI_TIME_SINCE_LAST_UPDATE)->SetWindowText(strItem);
   }

   //-- Update status bits
   { 
      //-- Regard status bits valid if ...
      //..     printer status is valid 
      //.. AND age less than ... ms 
      if (psPrinterStatus && (dwAgeSec < 99999)) {
         if (psPrinterStatus->dwDataValidFlag & D_EXO_PMP_PST_DVF_PRINTER_STATUS) 
              strItem.Format("[%8.8x]", psPrinterStatus->sPrinterStatus.dwStatus);
         else strItem.Format("<UNDEFINED>");
      } 
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_SI_STATUS_BITS)->SetWindowText(strItem);
   }

   //-- Update job identifier
   { 
      //-- Regard job identifier valid if ...
      //..     printer status is valid  
      //.. AND age less than ... ms 
      if (psPrinterStatus && (dwAgeSec < 99999)) {
         if (psPrinterStatus->dwDataValidFlag & D_EXO_PMP_PST_DVF_PRINTER_STATUS) 
              strItem.Format("%d", (WORD) psPrinterStatus->sPrinterStatus.ubPrintJobId);
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_SI_JOB_IDENTIFIER)->SetWindowText(strItem);
   }

   //-- Update print head temperature
   { 
      //-- Regard print head temperature valid if ...
      //..     printer status is valid  
      //.. AND age less than ... ms 
      if (psPrinterStatus && (dwAgeSec < 99999)) {
         if (psPrinterStatus->dwDataValidFlag & D_EXO_PMP_PST_DVF_PRINTER_STATUS) 
              strItem.Format("%+d C", (INT) psPrinterStatus->sPrinterStatus.siPrintHeadTempC);
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_SI_PH_TEMPERATURE)->SetWindowText(strItem);
   }

   //-- Update board voltage
   { 
      //-- Regard board voltage valid if ...
      //..     printer status is valid  
      //.. AND age less than ... ms 
      if (psPrinterStatus && (dwAgeSec < 99999)) {
         if (psPrinterStatus->dwDataValidFlag & D_EXO_PMP_PST_DVF_PRINTER_STATUS) 
              strItem.Format("%2.1lf V", (double) psPrinterStatus->sPrinterStatus.wBoardVoltageTv / 10);
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_SI_BOARD_VOLTAGE)->SetWindowText(strItem);
   }

   //-- Update printer state
   { 
      //-- Regard printer state valid if ...
      //..     printer status is valid 
      //.. AND age less than ... ms 
      if (psPrinterStatus && (dwAgeSec < 99999)) {
         if (psPrinterStatus->dwDataValidFlag & D_EXO_PMP_PST_DVF_PRINTER_STATUS) 
              strItem.Format("%d", (WORD) psPrinterStatus->sPrinterStatus.ubState);
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_SI_PRINTER_STATE)->SetWindowText(strItem);
   }

   //-- Update printer error
   { 
      //-- Regard printer error if ...
      //..     printer status is valid 
      //.. AND age less than ... ms 
      if (psPrinterStatus && (dwAgeSec < 99999)) {
         if (psPrinterStatus->dwDataValidFlag & D_EXO_PMP_PST_DVF_PRINTER_STATUS) 
              strItem.Format("%d", (WORD) psPrinterStatus->sPrinterStatus.wError);
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_SI_PRINTER_ERROR)->SetWindowText(strItem);
   }
}

//******************************************************************************************
//** METHOD    CExo_Pmpt_DlgMain::UpdateDialog
//** ABSTRACT  Retrieves registry data supplied by print monitor plugins and updates the 
//**           corresponding dialog items
//******************************************************************************************

void CExo_Pmpt_DlgMain::UpdateDialog
   ()
{
   //-- Read job information from registry and update dialog 
   { 
      TS_EXO_PMP_JOB_INFO sJobInfo;  // Job information structure as stored in registry

      //-- Read job info from registry and check revision
      if (     ReadPrinterDataFromRegistry(m_pszPrinter, D_EXO_PMP_REG_KEY, D_EXO_PMP_REG_VALUE_PMP_JOB_INFO, (PBYTE) &sJobInfo, sizeof(sJobInfo))
           && (sJobInfo.ubRevision == D_EXO_PMP_JI_REVISION)
         ) UpdateJobInfoInDialog(&sJobInfo);
      else UpdateJobInfoInDialog(NULL);
   }

   //-- Read printer information from registry and update dialog
   { 
      TS_EXO_PMP_PRINTER_INFO sPrinterInfo; // Printer information structure as stored in registry

      //-- Read printer info from registry and check revision
      if (     ReadPrinterDataFromRegistry(m_pszPrinter, D_EXO_PMP_REG_KEY, D_EXO_PMP_REG_VALUE_PMP_PRINTER_INFO, (PBYTE) &sPrinterInfo, sizeof(sPrinterInfo))
           && (sPrinterInfo.ubRevision == D_EXO_PMP_PI_REVISION)
         ) UpdatePrinterInfoInDialog(&sPrinterInfo);
      else UpdatePrinterInfoInDialog(NULL);
   }

   //-- Read status information from registry and update dialog
   { 
      TS_EXO_PMP_PRINTER_STATUS sPrinterStatus; // Status information structure as stored in registry

      //-- Read printer info from registry and check revision
      if (     ReadPrinterDataFromRegistry(m_pszPrinter, D_EXO_PMP_REG_KEY, D_EXO_PMP_REG_VALUE_PMP_PRINTER_STATUS, (PBYTE) &sPrinterStatus, sizeof(sPrinterStatus))
           && (sPrinterStatus.ubRevision == D_EXO_PMP_PST_REVISION)
         ) UpdatePrinterStatusInDialog(&sPrinterStatus);
      else UpdatePrinterStatusInDialog(NULL);
   }
}

